use Test::More;

use_ok( 'POEx::Role::SessionInstantiation::Meta::Session::Magic' );
use_ok( 'POEx::Role::SessionInstantiation::Meta::Session::Events' );
use_ok( 'POEx::Role::SessionInstantiation::Meta::Session::Sugar' );
use_ok( 'POEx::Role::SessionInstantiation::Meta::Session::Implementation' );
use_ok( 'POEx::Role::SessionInstantiation' );

done_testing();
