package POEx::Role::Event;
$POEx::Role::Event::VERSION = '1.100920';

#ABSTRACT: Provide a decorator to label events

use MooseX::Declare;

role POEx::Role::Event
{
}
1;


=pod

=head1 NAME

POEx::Role::Event - Provide a decorator to label events

=head1 VERSION

version 1.100920

=head1 AUTHOR

  Nicholas Perez <nperez@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2010 by Nicholas Perez.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut


__END__
